#!/usr/bin/env python
#
# fio-beacon.py		Emulate Fusion-io fio-beacon command-line utility
#

import sys
if sys.hexversion < 0x02060500:
    print >> sys.stderr, "This script requires python 2.6.5 or higher."
    exit(1)

try:
    import argparse
except ImportError:
    print >> sys.stderr, "Please install argparse from http://code.google.com/p/argparse."
    exit(1)

try:
    import pywbem
except ImportError:
    print >> sys.stderr, "Please install the pywbem module from http://sourceforge.net/projects/pywbem." 
    exit(1)

import string

__author__ = "Fusion-io management software team"
__date__ ="$Feb 15, 2011 1:42:17 PM$"
__version__ = "3.1.4.111 tahoe@a2a30d5308e5M"

def make_wbem_url(args):
    return "{0}://{1}".format('https' if args.secure_flag else 'http', args.server)

def beacon(conn, ioport_name, enable):
    return conn.InvokeMethod('Beacon', ioport_name, enable=enable)

def get_pci_bus_id(pci):
    return pci.properties['DeviceID'].value

# ---------------------------------------------------------------------------
# START OF MAIN

# Setup argparse data structures and parse command-line arguments

description = '''Fusion-io %(prog)s utility (3.1.4.111 tahoe@a2a30d5308e5M).
Copyright 2006-2011 Fusion-io, Inc. All rights reserved.'''

parser = argparse.ArgumentParser(prog='fio-beacon', description=description, formatter_class=argparse.RawDescriptionHelpFormatter)
parser.add_argument('device', metavar='<device>', nargs=1, help='a pathname to the device (/dev/fctX)');
parser.add_argument('-v', '--version', action='version', version='%(prog)s version 3.1.4.111 tahoe@a2a30d5308e5M', help="print version and exit")
parser.add_argument('-1', '--on', action='store_true', dest='enable_flag', default=True, help='enable beacon of device <dev>')
parser.add_argument('-0', '--off', action='store_false', dest='enable_flag', help='disable beacon of device <dev>')
parser.add_argument('-p', '--ppci', action='store_true', dest='print_pci_flag', help='print the pci bus id of device <dev>')

wbemgroup = parser.add_argument_group('remote options')
wbemgroup.add_argument('--server', metavar='<address>', dest='server', required=True, help='remote host address/dns name')
wbemgroup.add_argument('--namespace', metavar='<namespace>', dest='namespace', default='fio/cimv2', help='CIM namespace (default:fio/cimv2)')
wbemgroup.add_argument('--username', metavar='<username>', dest='username', default='anonymous', help='remote user (default:anonymous)')
wbemgroup.add_argument('--password', metavar='<password>', dest='password', default='', help="remote user's password (default:none)")
wbemgroup.add_argument('--no-ssl', default=True, action='store_false', dest='secure_flag', help='disable secure http on connection');

args = parser.parse_args()

# parse path name (/dev/fctx); extract fctx

short_device = string.split(args.device[0], '/')[-1]

# build a URL and credentials tuple from command line arguments

url = make_wbem_url(args)
creds = (args.username, args.password) if args.secure_flag else None

# establish connection from the URL

conn = pywbem.WBEMConnection(url, creds, default_namespace=args.namespace);

# enumerate all FIO_IoMemoryPort instances; look for target device

device_name = None
iomport_inames = conn.EnumerateInstanceNames('FIO_IoMemoryPort')
for iname in iomport_inames:
    if iname.keybindings['DeviceID'] == short_device:
        device_name = iname
        break

if device_name == None:
    print >> sys.stderr, "Error: Unable to locate device '%s'." % args.device[0]
    exit(1)

# gather data, send request

print "Sending 'beacon %sable' request to %s for device '%s'." \
        % ('en' if args.enable_flag else 'dis', url, args.device[0])

# request and display pci device id

if args.print_pci_flag:
    pci = None
    pci_instances = conn.EnumerateInstances('FIO_PCIDevice')
    for instance in pci_instances:
        if instance.properties['InstanceID'].value == 'Fusion-io:{0}:pci'.format(short_device):
            pci = instance
            break

    if pci != None:
        print "PCI address: %s" % get_pci_bus_id(pci)

# send beacon enable request

rv = beacon(conn, iname, args.enable_flag)
if not rv[0]:
    print >> sys.stderr, "Beacon request failed."
else:
    print '%s beacon %s' % (args.device[0], 'ON' if args.enable_flag else 'OFF')

# END OF MAIN
# ---------------------------------------------------------------------------
