/*-----------------------------------------------------------------------------
 |- Copyright 2006-2010 Fusion-io, Inc. All rights reserved.

 No use, or distribution, of this source code is permitted in any form or
 means without a valid, written license agreement with Fusion-io. Please
 refer to the included "License" or "License.txt" file for terms and
 conditions regarding the use and redistribution of this software.
-----------------------------------------------------------------------------*/

#ifndef _INCLUDE_FIO_PUBLIC_FUSION_SERVICE_TYPES_H_
#define _INCLUDE_FIO_PUBLIC_FUSION_SERVICE_TYPES_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <fio/public/common_types.h>
#include <fio/public/dyn_struct_alloc.h>

/******************************************
 *          Trim Service Types            *
 ******************************************/
typedef struct fio_trim_enable_req
{
    uint32_t    lib_version;    /**< Fusion-io SDK API version (from FIO_STRUCT_INIT) */
    uint8_t     trim_enabled;   /**< Set to non-zero value to enable trim, 0 to disable */
} fio_trim_enable_req_t;

typedef struct fio_trim_enable_rsp
{
    uint32_t    lib_version;     /**< Fusion-io SDK API version (from FIO_STRUCT_INIT) */
    uint8_t     setting_changed; /**< Whether or not trim was successfully configured to be enabled. */
    uint8_t     status_changed;  /**< Whether or not trimming was either started or stopped successfully. */
} fio_trim_enable_rsp_t;

typedef struct fio_trim_status
{
    uint32_t    lib_version;                /**< Fusion-io SDK API version (from FIO_STRUCT_INIT) */
    uint8_t     trim_enabled;               /**< Whether or not trim is configured as enabled.  This does not mean trim
                                                is currently active, it just means it is supposed to be. */
    uint8_t     trim_enabled_valid;         /**< Whether or not the trim_enabled was successfully retrieved. */
    uint8_t     os_trim_supported;          /**< Whether or not this OS supports built-in trim. */
    uint8_t     os_trim_supported_valid;    /**< Whether or not the os_trim_supported was successfully retrieved. */
    uint8_t     os_trim_active;             /**< Whether or not built-in OS trim is active. */
    uint8_t     os_trim_active_valid;       /**< Whether or not the os_trim_active was successfully retrieved. */
    uint8_t     fusion_trim_running;        /**< Whether or not the Fusion trim monitor service is running. */
    uint8_t     fusion_trim_running_valid;   /**< Whether or not the fusion_trim_running was successfully retrieved. */
} fio_trim_status_t;

#ifdef __cplusplus
}
#endif

#endif /* _INCLUDE_FIO_PUBLIC_FUSION_SERVICE_TYPES_H_ */
