/*-----------------------------------------------------------------------------
 |- Copyright 2006-2010 Fusion-io, Inc. All rights reserved.

 No use, or distribution, of this source code is permitted in any form or
 means without a valid, written license agreement with Fusion-io. Please
 refer to the included "License" or "License.txt" file for terms and
 conditions regarding the use and redistribution of this software.
-----------------------------------------------------------------------------*/

#ifndef _INCLUDE_FIO_PUBLIC_FUSION_SERVICES_H_
#define _INCLUDE_FIO_PUBLIC_FUSION_SERVICES_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <fio/public/fusion_service_types.h>

/**
 * \brief Activate or deactivate trim on this machine.  Trim provides a way for iodimms on this system to be notified
 *      when blocks on the disk are no longer in use, which can lead to significantly better performance.
 * \param input trim enable request.  See header (iodimm_types.h) for more information on how to populate this struct.
 * \return See the fio_iodimm_trim_enable_rsp_t declaration for return value descriptions.
 * \note This function is currently only implemented in Windows.
 * \par This function is independent of whether trim is implemented natively (as in Windows 7)
 *      or as a third-party service daemon.
 */
DllExport
fio_trim_enable_rsp_t
fio_set_trim_enabled(fio_trim_enable_req_t* input);

/**
 * \brief Get the configured and running status of trim on this system.  Trim provides a way for iodimms on this system 
 *      to be notified when blocks on the disk are no longer in use, which can lead to significantly better performance.
 * \param status trim status object, pre-allocated.  Will be populated by this function.
 * \note This function is currently only implemented in Windows.
 * \par This function is independent of whether trim is implemented natively (as in Windows 7)
 *      or as a third-party service daemon.
 */
DllExport
void
fio_get_trim_status(fio_trim_status_t* status);

#ifdef __cplusplus
}
#endif

#endif /* _INCLUDE_FIO_PUBLIC_FUSION_SERVICES_H_ */
