/*-----------------------------------------------------------------------------
 |- Copyright 2006-2010 Fusion-io, Inc. All rights reserved.

 No use, or distribution, of this source code is permitted in any form or
 means without a valid, written license agreement with Fusion-io. Please
 refer to the included "License" or "License.txt" file for terms and
 conditions regarding the use and redistribution of this software.
-----------------------------------------------------------------------------*/

#ifndef _INCLUDE_FIO_PUBLIC_VOL_MGMT_H_
#define _INCLUDE_FIO_PUBLIC_VOL_MGMT_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <fio/public/volume_mgmt_types.h>
#include <fio/public/iodimm_types.h>

/**
 * \brief Initializes the volume management functions (exported functions are listed here).
 */
DllExport 
fio_vol_mgmt_result_t 
fio_vol_mgmt_init(void);

/**
 * \brief Finalizes the volume management functions (exported functions are listed here).
 * This must be done to clean up resources that were allocated/loaded by fio_vol_mgmt_init.
 *
 * \note This API is currently only implemented on the Windows platform
 */
DllExport 
void 
fio_vol_mgmt_fini(void);

/**
 * \brief Opens a drive handle for a Fusion-io drive.
 * \pre selector must either previously been opened with open_device_handle or must have the name or index mode set.
 *
 * \note This API is currently only implemented on the Windows platform
 *
 * \param selector Device selector. See the header file (volume_mgmt_types.h) for more information on how to populate  
 *      this struct.  If the device was not previously opened via open_iodimm_handle, this function internally opens and 
 *      closes the device.
 * \param drive_handle Pointer to a pre-allocated drive handle where data will be stored.
 *
 * \return On success, FIO_VOLMAN_SUCCESS; otherwise, failure. The drive_handle->volumes are populated if the drive is part of one or more 
 *      volumes. In addition to other data, each volume will have one or more mount points if that volume has been 
 *      mounted. After a successful return, the caller must call fio_vol_mgmt_close_drive_handle to free any resources 
 *      held in the drive_handle.
 */
DllExport
fio_vol_mgmt_result_t 
fio_vol_mgmt_open_drive_handle(fio_iodimm_selector_t* selector,
                               fio_drive_handle_t* drive_handle);

/**
 * \brief Closes a drive handle for a Fusion-io drive.
 * \pre fio_vol_mgmt_open_drive_handle should have been previously called to initialize the member data properly.
 *
 * \note This API is currently only implemented on the Windows platform
 *
 * \param drive_handle Pointer to a fio_drive_handle_t that has been allocated and passed to
 *      fio_vol_mgmt_open_drive_handle.
 */
DllExport
void 
fio_vol_mgmt_close_drive_handle(fio_drive_handle_t* drive_handle);

/**
 * \brief Refreshes the volumes (and their data and mount points) held by drive_handle.
 * \pre drive_handle must must have been successfully opened using fio_vol_mgmt_open_drive_handle.
 *
 * \note This API is currently only implemented on the Windows platform
 *
 * \param drive_handle Pointer to a fio_drive_handle_t that has been allocated and passed to
 *      fio_vol_mgmt_open_drive_handle. Note that drive_handle may or may not contain existing volume information prior 
 *      to this function's invocation. In either case, fio_vol_mgmt_refresh_volumes takes care of freeing previously 
 *      allocated volume resources.
 * \return FIO_VOLMAN_SUCCESS on success, otherwise failure.
 */
DllExport
fio_vol_mgmt_result_t 
fio_vol_mgmt_refresh_volumes(fio_drive_handle_t* drive_handle);

/***** OS-neutral shared convenience functions *****/

/**
 * \brief Converts a fio_status_result_t to a fio_vol_mgmt_result_t
 */
fio_vol_mgmt_result_t 
status_result_to_vol_mgmt_result(fio_status_result_t s_result);


#ifdef __cplusplus
}
#endif

#endif /* _INCLUDE_FIO_PUBLIC_VOL_MGMT_H_ */

